import { dataSource } from "../data-source";
import { membershipStatus } from "../entity/membershipStatus";
import InternalException from "../exceptions/internalException";
import { membership } from "../entity/membership";

export default abstract class MembershipStatusService {
  /**
   * @description get all membershipStatuss
   * @returns {Promise<Array<membershipStatus>>}
   */
  static async getAll(): Promise<Array<membershipStatus>> {
    return await dataSource
      .getRepository(membershipStatus)
      .createQueryBuilder("membershipStatus")
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("membershipStatuss not found", err);
      });
  }

  /**
   * @description get membershipStatus by id
   * @returns {Promise<membershipStatus>}
   */
  static async getById(id: number): Promise<membershipStatus> {
    return await dataSource
      .getRepository(membershipStatus)
      .createQueryBuilder("membershipStatus")
      .where("membershipStatus.id = :id", { id: id })
      .getOneOrFail()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("membershipStatus not found by id", err);
      });
  }

  /**
   * @description get active memberships assigned to membershipStatus
   * @returns {Promise<Array<membership>>}
   */
  static async getMembersBymembershipStatusId(id: number): Promise<Array<membership>> {
    return await dataSource
      .getRepository(membershipStatus)
      .createQueryBuilder("membershipStatus")
      .leftJoinAndSelect("membershipStatus.memberships", "memberships")
      .leftJoinAndSelect("memberships.member", "member")
      .where("membershipStatus.id = :id", { id: id })
      .andWhere("membership.start <= :start", { start: new Date() })
      .andWhere("membership.end >= :end", { end: new Date() })
      .orWhere("membership.end IS NULL")
      .getOneOrFail()
      .then((res) => {
        return res.memberships;
      })
      .catch((err) => {
        throw new InternalException("membershipStatus assigned members not found by id", err);
      });
  }
}
